package edu.kit.scc.webreg.dto.service;

import java.util.ArrayList;
import java.util.List;

import edu.kit.scc.webreg.dao.BaseDao;
import edu.kit.scc.webreg.dao.project.ProjectDao;
import edu.kit.scc.webreg.dto.entity.ProjectEntityDto;
import edu.kit.scc.webreg.dto.mapper.BaseEntityMapper;
import edu.kit.scc.webreg.dto.mapper.ProjectEntityMapper;
import edu.kit.scc.webreg.entity.ServiceEntity;
import edu.kit.scc.webreg.entity.project.LocalProjectEntity;
import edu.kit.scc.webreg.entity.project.LocalProjectEntity_;
import edu.kit.scc.webreg.entity.project.ProjectEntity;
import edu.kit.scc.webreg.entity.project.ProjectEntity_;
import edu.kit.scc.webreg.entity.project.ProjectServiceEntity;
import jakarta.ejb.Stateless;
import jakarta.inject.Inject;

import edu.kit.scc.webreg.dto.mapper.ProjectIdentityAdminEntityMapper;
import edu.kit.scc.webreg.dto.mapper.ProjectMembershipEntityMapper;
import edu.kit.scc.webreg.dto.mapper.ProjectServiceEntityMapper;
import edu.kit.scc.webreg.service.project.LocalProjectService;
import edu.kit.scc.webreg.service.project.ProjectService;

@Stateless
public class ProjectDtoService extends BaseDtoServiceImpl<ProjectEntity, ProjectEntityDto> {

	private static final long serialVersionUID = 1L;

	@Inject
	private ProjectEntityMapper prEntitymapper;

	@Inject
	private ProjectIdentityAdminEntityMapper prIdAdminMapper;

	@Inject
	private ProjectMembershipEntityMapper prMembershipMapper;

	@Inject
	private ProjectServiceEntityMapper prServiceMapper;

	@Inject
	private LocalProjectService service;

	@Inject
	private ProjectService projectService;

	@Inject
	private ProjectDao dao;

	public List<ProjectEntityDto> findByService(ServiceEntity service) {
		List<ProjectServiceEntity> projectList = dao.findAllByService(service);
		List<ProjectEntityDto> dtoList = new ArrayList<ProjectEntityDto>(projectList.size());
		for (ProjectServiceEntity p : projectList) {
			ProjectEntityDto dto = createNewDto();
			prEntitymapper.copyProperties(p.getProject(), dto);
			dtoList.add(dto);
		}
		return dtoList;
	}

	public ProjectEntityDto findProjectById(Long projectId) {

		ProjectEntityDto pDto = createNewDto();

		LocalProjectEntity entity = service.findByIdWithAttrs(projectId, LocalProjectEntity_.projectServices,
				ProjectEntity_.childProjects);

		prEntitymapper.copyProperties(entity, pDto);

		pDto.setLinkedservices(prServiceMapper.getServiceList(entity.getProjectServices()));
		pDto.setAdmins(prIdAdminMapper.getAdminList(projectService.findAdminsForProject(entity)));
		pDto.setMembers(prMembershipMapper.getMemberList(projectService.findMembersForProject(entity)));

		return pDto;

	}

	@Override
	protected BaseEntityMapper<ProjectEntity, ProjectEntityDto> getMapper() {
		return prEntitymapper;
	}

	@Override
	protected BaseDao<ProjectEntity> getDao() {
		return dao;
	}

}
