package edu.kit.scc.webreg.job;

import static edu.kit.scc.webreg.dao.ops.PaginateBy.unlimited;
import static edu.kit.scc.webreg.dao.ops.RqlExpressions.lessThan;
import static edu.kit.scc.webreg.dao.ops.SortBy.ascendingBy;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.kit.scc.webreg.entity.identity.IdentityEmailAddressEntity;
import edu.kit.scc.webreg.entity.identity.IdentityEmailAddressEntity_;
import edu.kit.scc.webreg.service.identity.IdentityEmailAddressService;

public class ExpiredVerificationTokens extends AbstractExecutableJob {

	private static final long serialVersionUID = 1L;

	@Override
	public void execute() {
		Logger logger = LoggerFactory.getLogger(ExpiredVerificationTokens.class);

		try {
			InitialContext ic = new InitialContext();
			
			IdentityEmailAddressService service = (IdentityEmailAddressService) ic.lookup(
					"global/bwreg/bwreg-service/IdentityEmailAddressService!edu.kit.scc.webreg.service.identity.IdentityEmailAddressService");			
			
			List<IdentityEmailAddressEntity> expiredTokens = service.findAllEagerly(unlimited(),
					Arrays.asList(ascendingBy(IdentityEmailAddressEntity_.id)),
					lessThan(IdentityEmailAddressEntity_.tokenValidUntil, new Date()));

			if (expiredTokens != null && !expiredTokens.isEmpty()) {

				for (IdentityEmailAddressEntity em : expiredTokens) {
					service.deleteEmailAddress(em, "Expired token" );
				}

				logger.debug("Deletion is done");
			} else {
				logger.debug("No expired tokens to delete");
			}

		} catch (NamingException e) {
			logger.warn("Could not delete expired tokens: {}", e);
		}
   }
}