/*******************************************************************************
 * Copyright (c) 2014 Michael Simon.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Haykuhi Musheghyan - 2025
 ******************************************************************************/

package edu.kit.scc.webreg.bean;

import java.io.Serializable;
import java.util.Date;
import com.nimbusds.jwt.JWT;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.JWTParser;
import edu.kit.scc.webreg.entity.identity.IdentityEntity;
import edu.kit.scc.webreg.entity.oidc.OidcOpConfigurationEntity;
import edu.kit.scc.webreg.service.identity.IdentityService;
import edu.kit.scc.webreg.session.SessionManager;
import edu.kit.scc.webreg.util.FacesMessageGenerator;
import edu.kit.scc.webreg.util.ResourceBundleHelper;
import jakarta.faces.context.ExternalContext;
import jakarta.faces.context.FacesContext;
import jakarta.faces.view.ViewScoped;
import jakarta.inject.Inject;
import jakarta.inject.Named;
import jakarta.servlet.http.HttpServletRequest;
import lombok.Getter;
import lombok.Setter;

@Named
@ViewScoped
public class LoginViaAccessTokenBean implements Serializable {

	private static final long serialVersionUID = 1L;

	@Inject
	private SessionManager session;

	@Inject
	private HttpServletRequest request;

	@Inject
	private IdentityService identityService;

	@Inject
	private FacesMessageGenerator messageGenerator;

	@Inject
	private ResourceBundleHelper resourceHelper;

	private IdentityEntity identity;

	@Getter
	@Setter
	private OidcOpConfigurationEntity opConfig;

	@Getter
	@Setter
	private String tokenToValidate;

	public IdentityEntity getIdentity() {
		if (identity == null) {
			identity = identityService.fetch(session.getIdentityId());
		}
		return identity;

	}

	public void loginUsingAccessToken(String at) {
		try {

			JWT jwt = JWTParser.parse(at);
			JWTClaimsSet claims = jwt.getJWTClaimsSet();

			if (!claims.getIssuer().contains(request.getServerName())) {
				String details = resourceHelper.resolveMessage("token_issuer_details") + " " + claims.getIssuer();
				messageGenerator.addResolvedErrorMessage("failure", details, false);
			}

			Date expirationTime = claims.getExpirationTime();
			String aud = claims.getAudience().get(0);

			if (expirationTime != null && expirationTime.before(new Date())) {
				String details = resourceHelper.resolveMessage("token_expired_details") + " " + expirationTime;
				messageGenerator.addResolvedErrorMessage("failure", details, false);

			} else if (aud.contains("login")) {
				session.setIdentityId(Long.parseLong(claims.getSubject()));
				ExternalContext context = FacesContext.getCurrentInstance().getExternalContext();
				context.redirect(session.getOriginalRequestPath());
			} else {
				messageGenerator.addResolvedErrorMessage("failure", "token_scope_details", true);
			}

		} catch (Exception e) {
			messageGenerator.addResolvedErrorMessage("failure", "invalid_token_details", true);
		}

	}

}
