/*******************************************************************************
 * Copyright (c) 2014 Michael Simon.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Michael Simon - initial
 ******************************************************************************/
package edu.kit.scc.webreg.bean.admin;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;

import jakarta.annotation.PostConstruct;
import jakarta.enterprise.context.RequestScoped;
import jakarta.inject.Inject;
import jakarta.inject.Named;

import edu.kit.scc.webreg.entity.AdminUserEntity;
import edu.kit.scc.webreg.service.AdminUserService;
import edu.kit.scc.webreg.service.reg.PasswordUtil;
import edu.kit.scc.webreg.util.FacesMessageGenerator;
import edu.kit.scc.webreg.util.ViewIds;

@Named("addAdminUserBean")
@RequestScoped
public class AddAdminUserBean implements Serializable {

	private static final long serialVersionUID = 1L;

	@Inject
	private AdminUserService service;

	private AdminUserEntity entity;
	
	@Inject
	private FacesMessageGenerator messageGenerator;
	
	@Inject
	private PasswordUtil passwordUtil;
	
	@PostConstruct
	public void init() {
		entity = service.createNew();
	}
	
	public String save() {
		try {
			String hash = passwordUtil.generatePassword("SHA-512/256", entity.getPassword());
			entity.setPassword(hash);
		} catch (NoSuchAlgorithmException e) {
			messageGenerator.addResolvedErrorMessage("failure",
					"failure_details", true);
		} catch (UnsupportedEncodingException e) {
			messageGenerator.addResolvedErrorMessage("failure",
					"failure_details", true);
		}
		entity = service.save(entity);
		return ViewIds.SHOW_ADMIN_USER + "?id=" + entity.getId() + "&faces-redirect=true";
	}

	public AdminUserEntity getEntity() {
		return entity;
	}

	public void setEntity(AdminUserEntity entity) {
		this.entity = entity;
	}
}
