/*******************************************************************************
 * Copyright (c) 2014 Michael Simon.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Michael Simon - initial
 ******************************************************************************/
package edu.kit.scc.webreg.bean.admin;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.security.Provider.Service;
import java.security.Security;
import java.util.ArrayList;
import java.util.List;

import jakarta.faces.event.ComponentSystemEvent;
import jakarta.faces.view.ViewScoped;
import jakarta.inject.Inject;
import jakarta.inject.Named;
import lombok.Getter;
import lombok.Setter;

import org.primefaces.event.TransferEvent;
import org.primefaces.model.DualListModel;
import org.slf4j.Logger;

import edu.kit.scc.webreg.entity.AdminUserEntity;
import edu.kit.scc.webreg.entity.AdminUserEntity_;
import edu.kit.scc.webreg.entity.RoleEntity;
import edu.kit.scc.webreg.service.AdminUserService;
import edu.kit.scc.webreg.service.RoleService;
import edu.kit.scc.webreg.service.reg.PasswordUtil;

@Named
@ViewScoped
public class ShowAdminUserBean implements Serializable {

	private static final long serialVersionUID = 1L;

	@Inject
	private Logger logger;

	@Inject
	private AdminUserService adminUserService;

	@Inject
	private RoleService roleService;

	@Inject
	private PasswordUtil passwordUtil;

	@Getter
	@Setter
	private AdminUserEntity entity;

	@Getter
	@Setter
	private DualListModel<RoleEntity> roleList;

	@Getter
	@Setter
	private Long id;

	@Getter
	@Setter
	private String newPassword;
	
	@Getter
	@Setter
	private String[] hashMethod;
	
	@Getter
	@Setter
	private String selectedHashMethod;
	
	@Getter
	@Setter
	private boolean selected = true;

	public void preRenderView(ComponentSystemEvent ev) {
		if (entity == null) {
			entity = adminUserService.findByIdWithAttrs(id, AdminUserEntity_.roles);
			roleList = new DualListModel<RoleEntity>();

			List<RoleEntity> targetList = new ArrayList<RoleEntity>(entity.getRoles());
			List<RoleEntity> sourceList = roleService.findAll();

			sourceList.removeAll(targetList);

			roleList.setSource(sourceList);
			roleList.setTarget(targetList);

			fillHashMethod();
			selectedHashMethod = hashMethod[0];
			setSelected(true);
		}
	}

	protected void fillHashMethod() {
		Provider provider = Security.getProvider("BC");

		if (provider == null) {
			Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
			provider = Security.getProvider("BC");
		}

		List<String> algoList = new ArrayList<String>();

		for (Service service : provider.getServices()) {
			if (service.getType().equals("MessageDigest")) {
				if (service.getAlgorithm().equals("SHA-512/256"))
					algoList.add(0, service.getAlgorithm());
				else 
					algoList.add(service.getAlgorithm());
			}
		}

		hashMethod = algoList.toArray(new String[] {});
	}

	public void onTransfer(TransferEvent event) {

		if (event.isAdd()) {
			for (Object o : event.getItems()) {
				RoleEntity role = (RoleEntity) o;
				entity.getRoles().add(role);
				entity = adminUserService.save(entity);
			}
		} else {
			for (Object o : event.getItems()) {
				RoleEntity role = (RoleEntity) o;
				entity.getRoles().remove(role);
				entity = adminUserService.save(entity);
			}
		}
		entity = adminUserService.findByIdWithAttrs(id, AdminUserEntity_.roles);
	}

	public void savePassword() {
		if (newPassword != null) {
			newPassword = newPassword.trim();

			if (selected) {
				try {
					String hash = passwordUtil.generatePassword(selectedHashMethod, newPassword);
					entity.setPassword(hash);
				} catch (NoSuchAlgorithmException e) {
					logger.warn("Oh no", e);
				} catch (UnsupportedEncodingException e) {
					logger.warn("Oh no", e);
				}
			} else {
				entity.setPassword(newPassword);
			}
			entity = adminUserService.save(entity);
			entity = adminUserService.findByIdWithAttrs(id, AdminUserEntity_.roles);

			newPassword = "";
		}
	}

	public void handleSave() {
		entity = adminUserService.save(entity);
		entity = adminUserService.findByIdWithAttrs(id, AdminUserEntity_.roles);
	}
	
}
