/*******************************************************************************
 * Copyright (c) 2014 Michael Simon.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Haykuhi Musheghyan - initial
 ******************************************************************************/
package edu.kit.scc.webreg.bean.admin.project;

import static edu.kit.scc.webreg.dao.ops.RqlExpressions.equal;
import static edu.kit.scc.webreg.entity.UserEntity_.userStatus;
import static edu.kit.scc.webreg.entity.UserStatus.ACTIVE;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.primefaces.model.LazyDataModel;

import edu.kit.scc.webreg.bootstrap.LoadFromAppConfig;
import edu.kit.scc.webreg.entity.UserEntity;
import edu.kit.scc.webreg.entity.project.ExternalOidcProjectEntity;
import edu.kit.scc.webreg.entity.project.LocalProjectEntity;
import edu.kit.scc.webreg.entity.project.ProjectEntity;
import edu.kit.scc.webreg.entity.project.ProjectMembershipEntity;
import edu.kit.scc.webreg.entity.project.ProjectMembershipType;
import edu.kit.scc.webreg.model.GenericLazyDataModelImpl;
import edu.kit.scc.webreg.service.UserService;
import edu.kit.scc.webreg.service.project.ProjectService;
import edu.kit.scc.webreg.session.SessionManager;
import jakarta.faces.event.ComponentSystemEvent;
import jakarta.faces.view.ViewScoped;
import jakarta.inject.Inject;
import jakarta.inject.Named;
import lombok.Getter;
import lombok.Setter;

@Named
@ViewScoped
public class EditProjectMembersBean implements Serializable {

	private static final long serialVersionUID = 1L;

	@Inject
	private SessionManager session;

	@Inject
	private ProjectService projectService;

	@Inject
	private UserService userService;

	@Inject
	private LoadFromAppConfig config;

	@Getter
	@Setter
	private ProjectEntity entity;

	private List<ProjectMembershipEntity> membersList;

	private LazyDataModel<UserEntity> allUserList;

	@Getter
	@Setter
	private Long projectId;

	public void preRenderView(ComponentSystemEvent ev) {
		if (entity == null) {
			entity = projectService.fetch(projectId);
		}
	}

	public void addMember(UserEntity user) {

		ProjectMembershipEntity isMember = projectService.findByIdentityAndProject(user.getIdentity(), getEntity());

		if (isMember == null) {

			ProjectMembershipEntity pme = new ProjectMembershipEntity();
			pme.setMembershipType(ProjectMembershipType.MEMBER);

			config.getMemberShipValdity().ifPresent(validity -> pme
					.setMembershipValidity(new Date(System.currentTimeMillis() + validity)));

			projectService.addProjectMember(getEntity(), user.getIdentity(), pme, "idty-" + user.getIdentity().getId());

		}

		membersList = null;
	}

	public String cancel() {
		if (entity instanceof ExternalOidcProjectEntity)
			return "show-external-oidc-project.xhtml?faces-redirect=true&id=" + entity.getId();
		else if (entity instanceof LocalProjectEntity)
			return "show-local-project.xhtml?faces-redirect=true&id=" + entity.getId();
		else
			return "";
	}

	public List<ProjectMembershipEntity> getMembersList() {
		if (membersList == null) {
			membersList = projectService.findMembersForProject(getEntity());
		}
		return membersList;
	}

	public LazyDataModel<UserEntity> getAllUserList() {
		if (allUserList == null) {
			allUserList = new GenericLazyDataModelImpl<UserEntity, UserService>(userService, equal(userStatus, ACTIVE));
		}
		return allUserList;
	}

	public void deleteProjectMembership(ProjectMembershipEntity pme) {
		projectService.removeProjectMember(pme, "idty-" + pme.getIdentity().getId());
	}

}
