/*******************************************************************************
 * Copyright (c) 2014 Michael Simon.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Michael Simon - initial
 ******************************************************************************/
package edu.kit.scc.webreg.bean.admin.project;

import java.io.Serializable;
import java.util.List;

import edu.kit.scc.webreg.entity.project.LocalProjectEntity;
import edu.kit.scc.webreg.entity.project.LocalProjectEntity_;
import edu.kit.scc.webreg.entity.project.ProjectIdentityAdminEntity;
import edu.kit.scc.webreg.entity.project.ProjectMembershipEntity;
import edu.kit.scc.webreg.service.project.LocalProjectService;
import edu.kit.scc.webreg.service.project.ProjectService;
import edu.kit.scc.webreg.session.SessionManager;
import jakarta.faces.event.ComponentSystemEvent;
import jakarta.faces.view.ViewScoped;
import jakarta.inject.Inject;
import jakarta.inject.Named;
import lombok.Getter;
import lombok.Setter;

@Named("admin.showLocalProjectBean")
@ViewScoped
public class ShowLocalProjectBean implements Serializable {

	private static final long serialVersionUID = 1L;

	@Inject
	private LocalProjectService service;

	@Inject
	private ProjectService projectService;

	@Inject
	private SessionManager session;
	
	@Setter
	private LocalProjectEntity entity;
	
	private List<ProjectMembershipEntity> memberList;
	
	private List<ProjectIdentityAdminEntity> adminList;

	@Getter
	@Setter
	private Long id;

	public void preRenderView(ComponentSystemEvent ev) {
	}

	public void syncAllMembersToGroup() {
		projectService.syncAllMembersToGroup(getEntity(), "idty-" + session.getIdentityId());
	}

	public void triggerGroupUpdate() {
		projectService.triggerGroupUpdate(getEntity(), "idty-" + session.getIdentityId());
	}
	
	public LocalProjectEntity getEntity() {
		if (entity == null) {
			entity = service.findByIdWithAttrs(id, LocalProjectEntity_.projectServices);
		}

		return entity;
	}

	public List<ProjectMembershipEntity> getMemberList() {
		if (memberList == null) {
			memberList = projectService.findMembersForProject(getEntity());
		}
		return memberList;
	}

	public List<ProjectIdentityAdminEntity> getAdminList() {
		if (adminList == null) {
			adminList = projectService.findAdminsForProject(getEntity());
		}
		return adminList;
	}
}
