package edu.kit.scc.webreg.rest;

import java.util.List;

import edu.kit.scc.webreg.drools.exc.UnauthorizedException;
import edu.kit.scc.webreg.dto.entity.ProjectEntityDto;
import edu.kit.scc.webreg.dto.service.ProjectDtoService;
import edu.kit.scc.webreg.entity.project.LocalProjectEntity;
import edu.kit.scc.webreg.entity.project.LocalProjectEntity_;
import edu.kit.scc.webreg.entity.project.ProjectEntity_;
import edu.kit.scc.webreg.entity.project.ProjectIdentityAdminEntity;
import edu.kit.scc.webreg.exc.NoItemFoundException;
import edu.kit.scc.webreg.service.project.LocalProjectService;
import edu.kit.scc.webreg.service.project.ProjectService;
import edu.kit.scc.webreg.util.StringIds;
import jakarta.inject.Inject;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.Context;


@Path("/project")
public class UserProjectsController {

	@Inject
	private ProjectDtoService prDtoService;

	@Inject
	private LocalProjectService service;

	@Inject
	private ProjectService projectService;

	@Path(value = "/byid/{id}")
	@Produces({ "application/json" })
	@GET
	public ProjectEntityDto dumpAllInfoProProject(@PathParam("id") long id, @Context HttpServletRequest request)
			throws NoItemFoundException, UnauthorizedException {
				
		LocalProjectEntity entity = service.findByIdWithAttrs(id, LocalProjectEntity_.projectServices,
				ProjectEntity_.childProjects);
			
		if (entity == null)
			throw new NoItemFoundException("No project found");
		
		List<ProjectIdentityAdminEntity> admins = projectService.findAdminsForProject(entity);
				
		if (admins == null)
			throw new NoItemFoundException("No admins found");

		
		boolean allow = admins.stream().anyMatch(adm -> checkAccess(request, adm.getIdentity().getId()));

		if (!allow) {
			throw new UnauthorizedException("No access");
		}
		
		
		ProjectEntityDto obj = prDtoService.findProjectById(id);
		if (obj == null)
			throw new NoItemFoundException("No project info is available");
	
		return obj;

	}

	protected Boolean checkAccess(HttpServletRequest request, Long id) {	
		
	    Object identityIdObj = request.getAttribute(StringIds.IDENTITY_ID);
	    if (identityIdObj instanceof Long && id.equals(identityIdObj)) {
	        return Boolean.TRUE;
	    }
	    return Boolean.FALSE;
	}
	

}
