package edu.kit.scc.webreg.bootstrap;

import java.io.Serializable;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import jakarta.inject.Named;


@Named("loadFromAppConfig")
@ApplicationScoped
public class LoadFromAppConfig  implements Serializable {
	private static final long serialVersionUID = 1L;
	
	@Inject
	private ApplicationConfig appConfig;
	
	public Optional<Long> getMemberShipValdity() {

		String membershipValidityFromAppConfig = appConfig.getConfigValue("membership_validity");
		
		if (membershipValidityFromAppConfig == null)
			return Optional.empty();
		
		String regex = "^\\d{11}$";

		Pattern pattern = Pattern.compile(regex);
		Matcher matcher = pattern.matcher(membershipValidityFromAppConfig);

		if (matcher.matches()) {
			long membershipValidityMs;
			try {
				membershipValidityMs = Long.parseLong(membershipValidityFromAppConfig);
				return Optional.of(membershipValidityMs);

			} catch (NumberFormatException e) {
				throw new IllegalArgumentException(
						"Invalid format for membership_validity in config: " + membershipValidityFromAppConfig, e);
			}
		}
		
		return Optional.empty();
	}
	
	public Optional<Long> getProjectValdity() {

		String projectValidityFromAppConfig = appConfig.getConfigValue("project_validity");
		
		if (projectValidityFromAppConfig == null)
			return Optional.empty();
		
		String regex = "^\\d{11}$";

		Pattern pattern = Pattern.compile(regex);
		Matcher matcher = pattern.matcher(projectValidityFromAppConfig);

		if (matcher.matches()) {
			long projectValidityMs;
			try {
				projectValidityMs = Long.parseLong(projectValidityFromAppConfig);
				return Optional.of(projectValidityMs);

			} catch (NumberFormatException e) {
				throw new IllegalArgumentException(
						"Invalid format for project_validity in config: " + projectValidityFromAppConfig, e);
			}
		} else
			return Optional.empty();

	}
	

}

