package edu.kit.scc.regapp.oidc.tools;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;

import com.nimbusds.jwt.JWT;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.JWTParser;
import com.nimbusds.oauth2.sdk.token.BearerAccessToken;
import com.nimbusds.oauth2.sdk.token.RefreshToken;
import com.nimbusds.openid.connect.sdk.claims.IDTokenClaimsSet;
import com.nimbusds.openid.connect.sdk.claims.UserInfo;

@ApplicationScoped
public class OidcTokenHelper {
	
	@Inject
	private Logger logger;

	public Map<String, List<Object>> convertToAttributeMap(IDTokenClaimsSet claims, UserInfo userInfo, RefreshToken refreshToken, BearerAccessToken bat) {
		Map<String, List<Object>> attributeMap = new HashMap<String, List<Object>>();
		
		List<Object> tempList;
		
		if (claims != null) {
			tempList = new ArrayList<Object>();
			tempList.add(claims);
			attributeMap.put("claims", tempList);
		}

		if (userInfo != null) {
			tempList = new ArrayList<Object>();
			tempList.add(userInfo);
			attributeMap.put("userInfo", tempList);
		}
		
		if (refreshToken != null) {
			tempList = new ArrayList<Object>();
			tempList.add(refreshToken.getValue());
			attributeMap.put("refreshToken", tempList);
		}
		
		if (bat != null) {
			tempList = new ArrayList<Object>();
			tempList.add(bat);
			attributeMap.put("bearerAccessToken", tempList);
		}
		
		return attributeMap;
	}
	
	public IDTokenClaimsSet claimsFromMap(Map<String, List<Object>> attributeMap) {
		IDTokenClaimsSet claims;
		if (attributeMap.containsKey("claims") && 
				attributeMap.get("claims").get(0) instanceof IDTokenClaimsSet) {
			claims = (IDTokenClaimsSet) attributeMap.get("claims").get(0);
			return claims;
		}
		else {
			return null;
		}
	}
	
	public UserInfo userInfoFromMap(Map<String, List<Object>> attributeMap) {
		UserInfo userInfo;
		if (attributeMap.containsKey("userInfo") && 
				attributeMap.get("userInfo").get(0) instanceof UserInfo) {
			userInfo = (UserInfo) attributeMap.get("userInfo").get(0);
			return userInfo;
		}
		else {
			return null;
		}
	}
	
	public Long checkTokenValidityAndGetIdentityId(String path, String at, String host) {
		try {

			JWT jwt = JWTParser.parse(at);
			JWTClaimsSet claims = jwt.getJWTClaimsSet();

			if (!claims.getIssuer().contains(host)) {
				logger.error("Wrong token issuer {}. It should be contain the correct host name, i.e ", host );
				return 0L;
			}

			Date expirationTime = claims.getExpirationTime();
			Date currentTime = new Date();

			if (expirationTime != null && expirationTime.before(currentTime)) {
				logger.error("The token expired on " + expirationTime);
				return 0L;

			} else {
				String aud = claims.getAudience().get(0);
				if ((path.contains("/rest/project") && aud.contains("/rest/project")) || 
					    (path.contains("/login") && aud.contains("/login"))) {		
					logger.info("Correct token and it is valid until " + expirationTime);
					return Long.parseLong(claims.getSubject());
				}		
				
				else
					return 0L;
				
			}

		} catch (Exception e) {
			logger.error("Exception occured: ", e);		
			return 0L;
		}

	}
	
}
